import postcss from 'postcss';
interface CssToVariableOptions {
    /** 要扫描的目录路径 */
    directory: string;
    /** 要提取的CSS属性列表 */
    properties: string[];
    /** 变量名前缀 */
    prefix?: string;
    /** 输出的变量文件名 */
    outputFile?: string;
    /** 文件匹配模式 */
    pattern?: string;
    /** 自定义变量命名规则 */
    nameFormatter?: (property: string, value: string, decl?: postcss.Declaration) => string;
    /** 是否导出变量映射关系 */
    exportMap?: boolean;
    /** 是否导出图片资源base64 */
    assetsOutput?: boolean;
    /** 是否按文件夹拆分变量文件 */
    splitByFolder?: boolean;
}
interface VariableUsage {
    filePath: string;
    line: number;
    property: string;
    value: string;
}
interface VariableReport {
    variableName: string;
    value: string;
    usageCount: number;
    usages: VariableUsage[];
}
export declare class CssToVariable {
    private options;
    private extractedVariables;
    private extractedAssets;
    private variableMap;
    constructor(options: CssToVariableOptions);
    /**
     * 默认变量命名规则
     */
    private defaultNameFormatter;
    /**
     * 生成变量名
     */
    private generateVariableName;
    /**
     * 更新变量使用统计
     */
    private updateVariableUsage;
    /**
     * 解析单个文件
     */
    private parseFile;
    /**
     * 生成变量定义文件
     */
    private generateVariablesFile;
    /**
     * 生成资源变量文件
     */
    private generateAssetsFile;
    /**
     * 执行变量提取
     */
    extract(): Promise<void>;
    /**
     * 获取变量使用报告
     */
    getVariableReport(): VariableReport[];
}
export {};
